var logUpdate = require('log-update');
var spinner = require('elegant-spinner');
var chalk = require('chalk');
var Promise = require('native-or-bluebird');
var promise_finally_1 = require('promise-finally');
var inquirer = require('inquirer');
var archy = require('archy');
function loader(promise, options) {
    var frame = spinner();
    var update = function () { return logUpdate(frame()); };
    var interval = setInterval(update, 50);
    function end() {
        clearInterval(interval);
        logUpdate.clear();
    }
    update();
    return promise_finally_1.default(Promise.resolve(promise), end)
        .catch(function (error) {
        console.log(chalk.red(error.name + ": " + error.message));
        if (options.verbose && 'stack' in error) {
            console.log(error.stack);
        }
        process.exit(1);
        return Promise.reject(error);
    });
}
exports.loader = loader;
function inquire(questions) {
    return new Promise(function (resolve) {
        inquirer.prompt(questions, resolve);
    });
}
exports.inquire = inquire;
function archifyDependencyTree(tree, options) {
    if (options === void 0) { options = {}; }
    var result = {
        label: tree.name,
        nodes: []
    };
    function traverse(result, tree) {
        var nodes = result.nodes;
        if (options.ambient) {
            for (var _i = 0, _a = Object.keys(tree.ambientDependencies); _i < _a.length; _i++) {
                var name_1 = _a[_i];
                nodes.push(traverse({
                    label: name_1,
                    nodes: []
                }, tree.ambientDependencies[name_1]));
            }
        }
        else {
            for (var _b = 0, _c = Object.keys(tree.devDependencies); _b < _c.length; _b++) {
                var name_2 = _c[_b];
                nodes.push(traverse({
                    label: name_2,
                    nodes: []
                }, tree.devDependencies[name_2]));
            }
            for (var _d = 0, _e = Object.keys(tree.dependencies); _d < _e.length; _d++) {
                var name_3 = _e[_d];
                nodes.push(traverse({
                    label: name_3,
                    nodes: []
                }, tree.dependencies[name_3]));
            }
        }
        return result;
    }
    var archyTree = traverse(result, tree);
    if (archyTree.nodes.length === 0) {
        archyTree.nodes.push(chalk.cyan('No dependencies'));
    }
    return archy(archyTree);
}
exports.archifyDependencyTree = archifyDependencyTree;
//# sourceMappingURL=cli.js.map